package carleton150.edu.carleton.carleton150;

import android.annotation.TargetApi;
import android.content.ComponentName;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.Parcelable;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;


/**
 * Class for managing user selecting or taking photos.
 * Taken from http://andreborud.com/android-photo-intent-chooser-cameragallery/
 */



    public class Camera {

       private static Constants constants = new Constants();

        private static boolean isExternalStorageDocument(Uri uri) {
            return "com.android.externalstorage.documents".equals(uri.getAuthority());
        }

        private static boolean isDownloadsDocument(Uri uri) {
            return "com.android.providers.downloads.documents".equals(uri.getAuthority());
        }

        private static boolean isMediaDocument(Uri uri) {
            return "com.android.providers.media.documents".equals(uri.getAuthority());
        }

        private static boolean isGooglePhotosUri(Uri uri) {
            return "com.google.android.apps.photos.content".equals(uri.getAuthority());
        }

    /**
     * Gets the file path for a give Uri
     *
     * @param context
     * @param uri
     * @return
     */
        @TargetApi(Build.VERSION_CODES.KITKAT)
        private static String getPath(final Context context, final Uri uri) {

            final boolean isKitKat = Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT;
            if (isKitKat && DocumentsContract.isDocumentUri(context, uri)) {
                if (isExternalStorageDocument(uri)) {
                    final String docId = DocumentsContract.getDocumentId(uri);
                    final String[] split = docId.split(":");
                    final String type = split[0];

                    if ("primary".equalsIgnoreCase(type)) {
                        return Environment.getExternalStorageDirectory() + "/" + split[1];
                    }
                }
                // DownloadsProvider
                else if (isDownloadsDocument(uri)) {

                    final String id = DocumentsContract.getDocumentId(uri);
                    final Uri contentUri = ContentUris.withAppendedId(
                            Uri.parse("content://downloads/public_downloads"), Long.valueOf(id));

                    return getDataColumn(context, contentUri, null, null);
                }
                // MediaProvider
                else if (isMediaDocument(uri)) {
                    final String docId = DocumentsContract.getDocumentId(uri);
                    final String[] split = docId.split(":");
                    final String type = split[0];

                    Uri contentUri = null;
                    switch (type) {
                        case "image":
                            contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
                            break;
                        case "video":
                            contentUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
                            break;
                        case "audio":
                            contentUri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
                            break;
                    }

                    final String selection = "_id=?";
                    final String[] selectionArgs = new String[] {
                            split[1]
                    };

                    return getDataColumn(context, contentUri, selection, selectionArgs);
                }
            }
            // MediaStore (and general)
            else if ("content".equalsIgnoreCase(uri.getScheme())) {

                // Return the remote address
                if (isGooglePhotosUri(uri))
                    return uri.getLastPathSegment();

                return getDataColumn(context, uri, null, null);
            }
            // File
            else if ("file".equalsIgnoreCase(uri.getScheme())) {
                return uri.getPath();
            }

            return null;
        }

        private static String getDataColumn(Context context, Uri uri,
                                            String selection, String[] selectionArgs) {

            Cursor cursor = null;
            final String column = "_data";
            final String[] projection = {
                    column
            };

            try {
                cursor = context.getContentResolver().query(uri, projection, selection, selectionArgs,
                        null);
                if (cursor != null && cursor.moveToFirst()) {

                    final int column_index = cursor.getColumnIndexOrThrow(column);
                    return cursor.getString(column_index);
                }
            } finally {
                if (cursor != null)
                    cursor.close();
            }
            return null;
        }

    /**
     * Gets the orientation of an image given a path to the image
     * @param imagePath
     * @return
     */
        private static int getImageOrientation(String imagePath){
            int rotate = 0;
            try {

                File imageFile = new File(imagePath);
                ExifInterface exif = new ExifInterface(
                        imageFile.getAbsolutePath());
                int orientation = exif.getAttributeInt(
                        ExifInterface.TAG_ORIENTATION,
                        ExifInterface.ORIENTATION_NORMAL);

                switch (orientation) {
                    case ExifInterface.ORIENTATION_ROTATE_270:
                        rotate = 270;
                        break;
                    case ExifInterface.ORIENTATION_ROTATE_180:
                        rotate = 180;
                        break;
                    case ExifInterface.ORIENTATION_ROTATE_90:
                        rotate = 90;
                        break;
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            return rotate;
        }

    /**
     * Given a BitmapFactory.Options and required width and height, determines
     * how much the Bitmap needs to be scaled down
     *
     * @param options
     * @param reqWidth
     * @param reqHeight
     * @return
     */
        private static int calculateInSampleSize(BitmapFactory.Options options,
                                                 int reqWidth, int reqHeight) {
            final int height = options.outHeight;
            final int width = options.outWidth;
            int inSampleSize = 1;

            if (height > reqHeight || width > reqWidth) {
                final int heightRatio = Math.round((float) height / (float) reqHeight);
                final int widthRatio = Math.round((float) width / (float) reqWidth);
                inSampleSize = heightRatio < widthRatio ? heightRatio : widthRatio;
            }

            return inSampleSize;
        }

    /**
     * Starts an intent to take a photo or select a photo from gallery
     * and puts the photo in the photoFile
     * @param context
     * @param photoFile file in which to store image
     * @return
     */
        public static Intent photoIntent(Context context, File photoFile) {
            List<Intent> cameraIntents = new ArrayList<>();
            Intent captureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
            if (photoFile != null)
                captureIntent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(photoFile));
            PackageManager packageManager = context.getPackageManager();
            final List<ResolveInfo> listCam = packageManager.queryIntentActivities(captureIntent, 0);
            for (ResolveInfo res : listCam) {
                String packageName = res.activityInfo.packageName;
                Intent intent = new Intent(captureIntent);
                intent.setComponent(
                        new ComponentName(res.activityInfo.packageName, res.activityInfo.name));
                intent.setPackage(packageName);
                cameraIntents.add(intent);
            }

            Intent galleryIntent = new Intent();
            galleryIntent.setType("image/*");
            galleryIntent.setAction(Intent.ACTION_GET_CONTENT);
            Intent chooserIntent = Intent.createChooser(galleryIntent, "Select Source");
            chooserIntent.putExtra(Intent.EXTRA_INITIAL_INTENTS,
                    cameraIntents.toArray(new Parcelable[cameraIntents.size()]));
            return chooserIntent;
        }

    /**
     * creates an image file for the image
     * @return
     * @throws IOException
     */
        public static File createImageFile() throws IOException {
            String dir = Environment.getExternalStorageDirectory() + File.separator
                    + "Pictures" + File.separator + "SomeFolder" + File.separator;
            File directory = new File(dir);
            if (directory.mkdir()) Log.d("dir created", dir);
            return new File(dir, constants.JPEG_FILE_PREFIX
                    + Calendar.getInstance().getTimeInMillis() + constants.JPEG_FILE_SUFFIX);
        }

        public static String photoResult(Context context, File file, Intent data) {
            String image;
            final boolean isCamera;
            if (data != null) {
                String action = data.getAction();
                if (action != null) Log.e("action", action);
                isCamera = action != null && action.equals("inline-data");
            } else isCamera = true;

            if(isCamera) {
                image = file.getAbsolutePath();
            } else {
                image = getPath(context, data.getData());
            }
            if (getImageOrientation(image) != 0) {
                BitmapFactory.Options options = new BitmapFactory.Options();
                options.inJustDecodeBounds = true;
                options.inSampleSize = calculateInSampleSize(options, 1024, 1024);
                options.inJustDecodeBounds = false;
                options.inPreferredConfig = Bitmap.Config.RGB_565;
                options.inDither = true;
                Bitmap bitmap = BitmapFactory.decodeFile(image, options);

                bitmap = getResizedBitmap(bitmap, 1024);
                Matrix matrix = new Matrix();
                matrix.postRotate(getImageOrientation(image));
                Bitmap rotatedBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(),
                        bitmap.getHeight(), matrix, true);
                FileOutputStream out = null;
                if (image != null) {
                    try {
                        out = new FileOutputStream(image);
                        rotatedBitmap.compress(Bitmap.CompressFormat.JPEG, 100, out);
                        bitmap.recycle();
                        rotatedBitmap.recycle();
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        try {
                            if (out != null) {
                                out.close();
                            }
                        } catch (Throwable ignore) {
                        }
                    }
                }
            }
            return image;

        }

    /**
     * Resizes the bitmap
     *
     * @param source
     * @param size
     * @return
     */
        private static Bitmap getResizedBitmap(Bitmap source, int size){

            if(source.getWidth()>size) {
                double ar = (double) source.getHeight() / (double) source.getWidth();

                int y = (int) (ar * size);
                Bitmap result = Bitmap.createScaledBitmap(source, size, y, false);
                if (result != source) {
                    source.recycle();
                }
                return result;
            }

            return source;
        }
    }

